package servlet;

import java.sql.*;

/*
 * Class for managing the database.
 */
public class Database implements AutoCloseable {
	private Connection conn = null;

	public Database() {
		try {
			conn = DriverManager
					.getConnection("jdbc:sqlite:lab.db3");

			// Display the contents of the database in the console.
			// This should be removed in the final version
			try (Statement stmt = conn.createStatement()) {
				ResultSet rs = stmt.executeQuery("select * from Respondents");
				while (rs.next()) {
					String name = rs.getString("name");
					System.out.println(name);
				}
			}

		} catch (SQLException e) {
			throw new RuntimeException(e);
		}
	}

	public boolean addName(String name) {
		String sql = "insert into Respondents (name) values(?)";

		try (PreparedStatement ps = conn.prepareStatement(sql)) {
			ps.setString(1, name);
			ps.executeUpdate();
			return true;
		} catch (SQLException e) {
			if (e.getErrorCode() == 1062 && e.getSQLState().equals("23000")) {
				// duplicate key error
				System.out.println(name + " already exists in the database");
			} else {
				printSqlError(e);
			}
			return false;
		}
		// Using the try-with-resources syntax, this will call ps.close() automatically
	}

	public boolean addProjectDetails(String name, String description) {
		String sql = "insert into ProjectDetails (projectName, projectDescription) values(?,?)";

		try (PreparedStatement ps = conn.prepareStatement(sql)) {
			ps.setString(1, name);
			ps.setString(2, description);
			ps.executeUpdate();
			return true;
		} catch (SQLException e) {
			if (e.getErrorCode() == 1062 && e.getSQLState().equals("23000")) {
				// duplicate key error
				System.out.println(name + " already exists in the database");
			} else {
				printSqlError(e);
			}
			return false;
		}
	}

	public boolean addAnswer(String name, String projectName, String answerOne, String answerTwo, String answerThree, String answerFour) {
		String sql = "insert into Answers (answerOne, answerTwo, answerThree, answerFour, name, projectName) values(?,?,?,?,?,?)";

		try (PreparedStatement ps = conn.prepareStatement(sql)) {
			ps.setString(1, answerOne);
			ps.setString(2, answerTwo);
			ps.setString(3, answerThree);
			ps.setString(4, answerFour);
			ps.setString(5, name);
			ps.setString(6, projectName);
			ps.executeUpdate();
			return true;
		} catch (SQLException e) {
			if (e.getErrorCode() == 1062 && e.getSQLState().equals("23000")) {
				// duplicate key error
				System.out.println(name + " already exists in the database");
			} else {
				printSqlError(e);
			}
			return false;
		}
	}

	/**
	 * Get all the survey results from the database
	 */
	public SurveyResult[] getSurveyResults() {
		String sql = "select * from SurveyResults";
		try (Statement stmt = conn.createStatement()) {
			ResultSet rs = stmt.executeQuery(sql);
			SurveyResult[] results = new SurveyResult[100];
			int i = 0;
			while (rs.next()) {
				String respName = rs.getString("name");
				String projectName = rs.getString("projectName");
				String projectDescription = rs.getString("projectDescription");
				
				int[] data = { rs.getInt("answerOne"), rs.getInt("answerTwo"), rs.getInt("answerThree"),
						rs.getInt("answerFour") };
				results[i] = new SurveyResult(respName, projectName, projectDescription, data);
				i++;
			}
			return results;
		} catch (SQLException e) {
			printSqlError(e);
			return null;
		}
	}

	private void printSqlError(SQLException e) {
		System.out.println("SQLException: " + e.getMessage());
		System.out.println("SQLState: " + e.getSQLState());
		System.out.println("VendorError: " + e.getErrorCode());
		throw new RuntimeException(e);
	}

	public void close() throws SQLException {
		conn.close();
	}
}
